@@ -1494,6 +1494,103 @@ class Test_Bucket(unittest.TestCase):
         bucket = self._make_one(properties=properties)
         self.assertEqual(bucket.project_number, PROJECT_NUMBER)
 
+    def test_retention_policy_effective_time_policy_missing(self):
+        bucket = self._make_one()
+        self.assertIsNone(bucket.retention_policy_effective_time)
+
+    def test_retention_policy_effective_time_et_missing(self):
+        properties = {
+            'retentionPolicy': {
+            },
+        }
+        bucket = self._make_one(properties=properties)
+
+        self.assertIsNone(bucket.retention_policy_effective_time)
+
+    def test_retention_policy_effective_time(self):
+        import datetime
+        from google.cloud._helpers import _datetime_to_rfc3339
+        from google.cloud._helpers import UTC
+
+        effective_time = datetime.datetime.utcnow().replace(tzinfo=UTC)
+        properties = {
+            'retentionPolicy': {
+                'effectiveTime': _datetime_to_rfc3339(effective_time),
+            },
+        }
+        bucket = self._make_one(properties=properties)
+
+        self.assertEqual(
+            bucket.retention_policy_effective_time, effective_time)
+
+    def test_retention_policy_locked_missing(self):
+        bucket = self._make_one()
+        self.assertFalse(bucket.retention_policy_locked)
+
+    def test_retention_policy_locked_false(self):
+        properties = {
+            'retentionPolicy': {
+                'isLocked': False,
+            },
+        }
+        bucket = self._make_one(properties=properties)
+        self.assertFalse(bucket.retention_policy_locked)
+
+    def test_retention_policy_locked_true(self):
+        properties = {
+            'retentionPolicy': {
+                'isLocked': True,
+            },
+        }
+        bucket = self._make_one(properties=properties)
+        self.assertTrue(bucket.retention_policy_locked)
+
+    def test_retention_period_getter_policymissing(self):
+        bucket = self._make_one()
+
+        self.assertIsNone(bucket.retention_period)
+
+    def test_retention_period_getter_pr_missing(self):
+        properties = {
+            'retentionPolicy': {
+            },
+        }
+        bucket = self._make_one(properties=properties)
+
+        self.assertIsNone(bucket.retention_period)
+
+    def test_retention_period_getter(self):
+        period = 86400 * 100  # 100 days
+        properties = {
+            'retentionPolicy': {
+                'retentionPeriod': str(period),
+            },
+        }
+        bucket = self._make_one(properties=properties)
+
+        self.assertEqual(bucket.retention_period, period)
+
+    def test_retention_period_setter_w_none(self):
+        period = 86400 * 100  # 100 days
+        bucket = self._make_one()
+        policy = bucket._properties['retentionPolicy'] = {}
+        policy['retentionPeriod'] = period
+
+        bucket.retention_period = None
+
+        self.assertIsNone(
+            bucket._properties['retentionPolicy']['retentionPeriod'])
+
+    def test_retention_period_setter_w_int(self):
+        period = 86400 * 100  # 100 days
+        bucket = self._make_one()
+
+        bucket.retention_period = period
+
+        self.assertEqual(
+            bucket._properties['retentionPolicy']['retentionPeriod'],
+            str(period))
+
     def test_self_link(self):
         SELF_LINK = 'http://example.com/self/'
         properties = {'selfLink': SELF_LINK}
@@ -2328,6 +2425,114 @@ class Test_Bucket(unittest.TestCase):
         with self.assertRaises(AttributeError):
             bucket.generate_upload_policy([])
 
+    def test_lock_retention_policy_no_policy_set(self):
+        credentials = object()
+        connection = _Connection()
+        connection.credentials = credentials
+        client = _Client(connection)
+        name = 'name'
+        bucket = self._make_one(client=client, name=name)
+        bucket._properties['metageneration'] = 1234
+
+        with self.assertRaises(ValueError):
+            bucket.lock_retention_policy()
+
+    def test_lock_retention_policy_no_metageneration(self):
+        credentials = object()
+        connection = _Connection()
+        connection.credentials = credentials
+        client = _Client(connection)
+        name = 'name'
+        bucket = self._make_one(client=client, name=name)
+        bucket._properties['retentionPolicy'] = {
+            'effectiveTime': '2018-03-01T16:46:27.123456Z',
+            'retentionPeriod': 86400 * 100,  # 100 days
+        }
+
+        with self.assertRaises(ValueError):
+            bucket.lock_retention_policy()
+
+    def test_lock_retention_policy_already_locked(self):
+        credentials = object()
+        connection = _Connection()
+        connection.credentials = credentials
+        client = _Client(connection)
+        name = 'name'
+        bucket = self._make_one(client=client, name=name)
+        bucket._properties['metageneration'] = 1234
+        bucket._properties['retentionPolicy'] = {
+            'effectiveTime': '2018-03-01T16:46:27.123456Z',
+            'isLocked': True,
+            'retentionPeriod': 86400 * 100,  # 100 days
+        }
+
+        with self.assertRaises(ValueError):
+            bucket.lock_retention_policy()
+
+    def test_lock_retention_policy_ok(self):
+        name = 'name'
+        response = {
+            'name': name,
+            'metageneration': 1235,
+            'retentionPolicy': {
+                'effectiveTime': '2018-03-01T16:46:27.123456Z',
+                'isLocked': True,
+                'retentionPeriod': 86400 * 100,  # 100 days
+            },
+        }
+        credentials = object()
+        connection = _Connection(response)
+        connection.credentials = credentials
+        client = _Client(connection)
+        bucket = self._make_one(client=client, name=name)
+        bucket._properties['metageneration'] = 1234
+        bucket._properties['retentionPolicy'] = {
+            'effectiveTime': '2018-03-01T16:46:27.123456Z',
+            'retentionPeriod': 86400 * 100,  # 100 days
+        }
+
+        bucket.lock_retention_policy()
+
+        kw, = connection._requested
+        self.assertEqual(kw['method'], 'POST')
+        self.assertEqual(kw['path'], '/b/{}/lockRetentionPolicy'.format(name))
+        self.assertEqual(kw['query_params'], {'ifMetagenerationMatch': 1234})
+
+    def test_lock_retention_policy_w_user_project(self):
+        name = 'name'
+        user_project = 'user-project-123'
+        response = {
+            'name': name,
+            'metageneration': 1235,
+            'retentionPolicy': {
+                'effectiveTime': '2018-03-01T16:46:27.123456Z',
+                'isLocked': True,
+                'retentionPeriod': 86400 * 100,  # 100 days
+            },
+        }
+        credentials = object()
+        connection = _Connection(response)
+        connection.credentials = credentials
+        client = _Client(connection)
+        bucket = self._make_one(
+            client=client, name=name, user_project=user_project)
+        bucket._properties['metageneration'] = 1234
+        bucket._properties['retentionPolicy'] = {
+            'effectiveTime': '2018-03-01T16:46:27.123456Z',
+            'retentionPeriod': 86400 * 100,  # 100 days
+        }
+
+        bucket.lock_retention_policy()
+
+        kw, = connection._requested
+        self.assertEqual(kw['method'], 'POST')
+        self.assertEqual(kw['path'], '/b/{}/lockRetentionPolicy'.format(name))
+        self.assertEqual(
+            kw['query_params'], {
+                'ifMetagenerationMatch': 1234,
+                'userProject': user_project,
+            })
+
 
 class _Connection(object):
     _delete_bucket = False
